({
    extendsFrom: "BaseField",
    rowId: 0,

    formRowTemplate: null,

    hasSystemName : false,

    notifiedOfParseError: false,

    fields: [],

    initialize: function(arguments){
        this._super("initialize", [arguments]);

        this.events = _.extend(this.events, {
            'click [px-context-action=add]': 'addRow',
            'click [px-context-action=remove]': 'removeRow',
            'click [px-context-action=add_to_editor]': 'addToEditor',
        });

        this.formRowTemplate = app.template.getField('lb-template-arguments', 'context-row', this.module);

        this.fields = [];
        this.notifiedOfParseError = false;
    },

    bindDataChange: function() {
        if (this.model) {
            this.model.on(/*"change:lb_template_name " + */"change:" + this.name, function(model, value) {
                this.render();
            }, this);
        }
    },

    _render: function(values) {

        // this._super('_render', [values])
        this.rowId = 0;
        this._loadTemplate();
        this.$el.html(this.template(this) || '');

        this.renderFields(values);
        // this.displayInsertButton();

    },

    displayInsertButton: function(){
        if(this.disposed || _.isEmpty(this.$el) || !this.$el.find('.px-context-actions').length)
            return;

        if(this.action == 'detail')
            this.$el.find('.px-context-actions').hide();
        else
            this.$el.find('.px-context-actions').show();
    },

    renderFields: function(values){
        var defs = values || this.model.get(this.name);

        if(!_.isEmpty(defs) && _.isString(defs)){
            try{
                defs = JSON.parse(defs);
            }catch(err){
                defs = null;
                var errorMsg = app.lang.get('LBL_CONTEXT_PARSE_ERROR', 'PX_NeedToDoEventTypes');
                app.logger.warn(errorMsg);
                if(!this.notifiedOfParseError) {
                    this.notifiedOfParseError = true;
                    app.alert.show('context_parse_error', {
                        level: 'warning',
                        messages: errorMsg
                    });
                }
            }
        }

        if(_.isEmpty(defs) || (_.isArray(defs) && defs.length <= 0) || (_.isString(defs) && defs == '[]')){
            this.renderRow(null, {});
            return;
        }

        var self = this;
        _.each(defs, function(data, key){
            self.renderRow(null, data);
        });
    },

    getRowSequenceNumber: function(data){
        if(_.isUndefined(data['key'])){
            return ++this.rowId;
        }

        if(data['key'] > this.rowId)
            this.rowId = data['key'];

        return data['key'];
    },

    addRow: function(e){
        var $row;
        if (e) {
            $row = this.$(e.currentTarget).closest('[data-container=px-context-row]');
            $row.after(this.formRowTemplate({hasSystemName: this.hasSystemName}));
            $row = $row.next();
        }

        this.renderRow($row, {});

        // var rowInput = $row.find('input[type="text"]');
        // if(rowInput.length)
        //     rowInput.focus();
    },

    removeRow: function(e){
        var $row = this.$(e.currentTarget).closest('[data-container=px-context-row]');

        this.removeModelValues($row);

        $row.remove();
        if (this.$('[data-container=px-context-row]').length === 0) {
            this.addRow();
        }
    },

    removeModelValues: function($row){
        var data = $row.data(),
            rowId = data['key'],
            labelName = this.getTemplateAttributeName(rowId),
            keyName = this.getAttributeSystemName(rowId);

        this.model.set(labelName, '');
        this.model.set(keyName, '');
    },

    renderRow: function($row, data){
        $row = $row || $(this.formRowTemplate()).appendTo(this.$el);
        data = data || {};

        var rowId = this.getRowSequenceNumber(data);

        this.renderTemplateAttribute($row, data, rowId);
        this.renderValueAttribute($row, data, rowId);

        $row.data('key', rowId);
    },

    renderTemplateAttribute: function($row, data, rowId){
        var field, fieldContainer, fieldEl,
            templateAttributeName = this.getTemplateAttributeName(rowId),
            templateAttributeValue = data.label || '',
            templateAttributeLabel = 'LBL_TPL_TEMPLATE_KEY';

        $row.data('key', templateAttributeValue);
        this.model.set(templateAttributeName, templateAttributeValue);
        var fieldDefs = _.extend({
            name: templateAttributeName,
            type: 'enum',
            // type: 'template-attributes-enum',
            selectPlaceholder: templateAttributeLabel,
            parentFieldName: this.name,
            argumentPart: 'template-attribute',
            disabled: true,
            layout: this
        }, this.def.options.label || {});
        field = this.createField(this.model, fieldDefs);
        fieldContainer = $row.find('[data-container=px-arguments-label]');
        fieldEl = $(field.getPlaceholder().string);
        fieldEl.appendTo(fieldContainer);

        this.renderField(field, fieldEl);
        this.fields.push(field);
    },

    renderValueAttribute: function($row, data, rowId){
        var fieldDefs, field,
            fieldContainer, fieldEl,
            fieldKeyName = this.getAttributeSystemName(rowId),
            fieldKeyValue = data.name || '',
            templateAttributeLabel = 'LBL_TPL_FIELD_NAME';

        $row.data('value', fieldKeyValue);
        this.model.set(fieldKeyName, fieldKeyValue);

        fieldDefs = _.extend({
            name: fieldKeyName,
            type: 'enum',
            selectPlaceholder: templateAttributeLabel,
            parentFieldName: this.name,
            argumentPart: 'target-field',
            disabled: true,
            layout: this
        }, this.def.options.name || {});

        field = this.createField(this.model, fieldDefs);
        fieldContainer = $row.find('[data-container=px-context-key]');
        fieldEl = $(field.getPlaceholder().string);
        fieldEl.appendTo(fieldContainer);

        this.renderField(field, fieldEl);
        this.fields.push(field);
    },

    getAttributeSystemName: function(key){
        return 'attribute_' + key
    },

    getTemplateAttributeName: function(key){
        return 'template_attribute_name_' + key
    },

    createField: function(model, def) {
        var obj = {
            meta: {
                view: 'edit'
            },
            def: def,
            model: model,
            context: app.controller.context,
            viewName: this.action,
            view: this.view
        };

        model.fields[def.name] = def;

        var field = app.view.createField(obj);
        field.action = this.action;
        return field;
    },

    renderField: function(field, $fieldEl) {
        field.setElement($fieldEl);
        try {
            field.render();
        } catch (e) {
            app.logger.error("Failed to render " + field + " on " + this.view + "\n" + e);
            app.error.handleRenderError(this.view, '_renderField', field);
        }
    },

    toggle: function(evt) {
        this.$('[data-container=px-context-row]').html('');
        this.renderFields();
    },

    clearErrorDecoration: function () {
        _.each(this.fields, function(field, key) {
            if(_.isEmpty(field) || _.isEmpty(field.$el)){
                // delete this.fields[key];
            }else {
                field.clearErrorDecoration();
            }
        }, this);

        this._super('clearErrorDecoration');
    },

    beforeHandleSave: function(){
        this.model.set('lb_template_arguments', this.getFormattedContextValue());
    },

    getContextFieldData: function(){
        var $rows = this.$('[data-container=px-context-row]');
        var contextData = [];

        _.each($rows, function(row) {
            var argumentItem = this.getArgument($(row));
            if(!_.isEmpty(argumentItem.name) && !_.isEmpty(argumentItem.label))
                contextData.push(argumentItem);
        }, this);

        return contextData;
    },

    getFormattedContextValue: function(){
        var contextData = this.getContextFieldData();
        return JSON.stringify(contextData, null);
    },

    getArgument: function($row) {
        var data = $row.data(),
            key = data['key'],
            fieldKeyName = this.getAttributeSystemName(key),
            fieldLabelName = this.getTemplateAttributeName(key);

        return {
            label: this.model.get(fieldLabelName),
            name: this.model.get(fieldKeyName),
            key: key
        };
    },

    _dispose: function() {
        _.each(this.fields, function(field){
            field.dispose();
        });
        this.fields = [];
        this._super('_dispose');
    },

    isFieldLengthValid: function(){
        if(_.isEmpty(this.model.get('lb_template_arguments')))
            return true;

        var contextField = btoa(escape(this.model.get('lb_template_arguments')));
        if(contextField.length >= 65535){
            return false;
        }
        return true;
    },

    getSymbolsAmountViolation: function(){
        var contextField = btoa(escape(this.model.get('lb_template_arguments')));
        return contextField.length - 65535;
    },

    substringContextFieldLength: function(){
        var contextList = this.getContextFieldData(),
            trimmedContextList = [],
            sortedContextList = [],
            whiteListedKeyNames = [],
            newContextList = [];

        sortedContextList = _.sortBy(contextList, function(context){ return context.label.length; });

        _.each(sortedContextList, function(context){
            var tempArgumentsJson = '';
            trimmedContextList.push(context);

            try{
                tempArgumentsJson = btoa(escape(JSON.stringify(trimmedContextList, null)));
                if(tempArgumentsJson.length <= 65535){
                    whiteListedKeyNames.push(context.name);
                }else{
                    return;
                }
            }catch(err){
                return;
            }
        }, this);


        if(!_.isEmpty(whiteListedKeyNames)){
            _.each(contextList, function(context, key){
                if(!_.isEmpty(context) && _.indexOf(whiteListedKeyNames, context.name) !== -1){
                    newContextList.push(context);
                }
            });
        }

        try{
            this.model.set('lb_template_arguments', JSON.stringify(newContextList, null));
        }catch(err){
            this.model.set('lb_template_arguments', '[]');
        }
    },

    getFieldsMeta: function(){
        var contextList = this.getContextFieldData(),
            fieldsMeta = [];

        _.each(contextList, function(contextItem){
            fieldsMeta.push({
                'name': 'template_attribute_name_' + contextItem.key
            });
        }, this);

        return fieldsMeta;
    },

})
